@@ -70,9 +70,15 @@ my %processByMetaFormat = (
 
 # data lengths for each INSV record type
 my %insvDataLen = (
-    0x300 => 56,
-    0x400 => 16,
-    0x700 => 53,
+    0x300 => 56,    # accelerometer
+    0x400 => 16,    # unknown
+    0x700 => 53,    # GPS
+);
+
+# limit the default amount of data we read for some record types
+# (to avoid running out of memory)
+my %insvLimit = (
+    0x300 => [ 'accelerometer', 20000 ],    # maximum of 20000 accelerometer records
 );
 
 # tags extracted from various QuickTime data streams
@@ -215,6 +221,10 @@ my %insvDataLen = (
             ByteOrder => 'Little-Endian',
         },
     }],
+    mett => { # Parrot drones
+        Name => 'mett',
+        SubDirectory => { TagTable => 'Image::ExifTool::Parrot::mett' },
+    },
     JPEG => { # (in CR3 images) - [vide HandlerType with JPEG in SampleDescription, not MetaFormat]
         Name => 'JpgFromRaw',
         Groups => { 2 => 'Preview' },
@@ -565,7 +575,7 @@ sub SaveMetaKeys($$$)
                 } else {
                     $str = '';
                 }
-                $et->VPrint(1, $$et{INDENT}."- Tag '".PrintableTagID($tag)."' ($len bytes)$str\n");
+                $et->VPrint(1, $$et{INDENT}."- Tag '".PrintableTagID($tag,2)."' ($len bytes)$str\n");
                 $et->VerboseDump(\$val);
             }
         }
@@ -794,6 +804,7 @@ sub ProcessSamples($)
                     $$et{ee} = $ee; # need ee information for 'keys'
                     $et->HandleTag($tagTbl, $metaFormat, undef,
                         DataPt  => \$buff,
+                        DataPos => $$start[$i],
                         Base    => $$start[$i],
                         TagInfo => $tagInfo,
                     );
@@ -815,6 +826,7 @@ sub ProcessSamples($)
                 # decode "freeGPS " data (Novatek)
                 ProcessFreeGPS($et, {
                     DataPt => \$buff,
+                    DataPos => $$start[$i],
                     SampleTime => $time[$i],
                     SampleDuration => $dur[$i],
                 }, $tagTbl) ;
@@ -827,6 +839,7 @@ sub ProcessSamples($)
                 FoundSomething($et, $tagTbl, $time[$i], $dur[$i]);
                 $et->HandleTag($tagTbl, $type, undef,
                     DataPt  => \$buff,
+                    DataPos => $$start[$i],
                     Base    => $$start[$i],
                     TagInfo => $tagInfo,
                 );
@@ -1392,7 +1405,7 @@ sub Process_mebx($$$)
 }
 
 #------------------------------------------------------------------------------
-# Process QuickTime '3gf' timed metadata (Pittasoft Blackvue dashcam)
+# Process QuickTime '3gf' timed metadata (ref PH, Pittasoft Blackvue dashcam)
 # Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
 # Returns: 1 on success
 sub Process_3gf($$$)
@@ -1419,32 +1432,6 @@ sub Process_3gf($$$)
     return 1;
 }
 
-#------------------------------------------------------------------------------
-# Process DuDuBell M1 dashcam / VSYS M6L 'gsen' atom (ref PH)
-# Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
-# Returns: 1 on success
-sub Process_gsen($$$)
-{
-    my ($et, $dirInfo, $tagTbl) = @_;
-    my $dataPt = $$dirInfo{DataPt};
-    my $dirLen = $$dirInfo{DirLen};
-    my $recLen = 3;     # 3-byte record length
-    $et->VerboseDir('gsen', undef, $dirLen);
-    if ($dirLen > $recLen and not $et->Options('ExtractEmbedded')) {
-        $dirLen = $recLen;
-        EEWarn($et);
-    }
-    my $pos;
-    for ($pos=0; $pos+$recLen<=$dirLen; $pos+=$recLen) {
-        $$et{DOC_NUM} = ++$$et{DOC_COUNT};
-        my @acc = map { $_ /= 16 } unpack "x${pos}c3", $$dataPt;
-        $et->HandleTag($tagTbl, Accelerometer => "@acc");
-        # (there are no associated timestamps, but these are sampled at 5 Hz in my test video)
-    }
-    delete $$et{DOC_NUM};
-    return 1;
-}
-
 #------------------------------------------------------------------------------
 # Process DuDuBell M1 dashcam / VSYS M6L 'gps0' atom (ref PH)
 # Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
@@ -1473,7 +1460,7 @@ sub Process_gps0($$$)
         $lat = $deg + ($lat - $deg * 100) / 60;
         $deg = int($lon / 100);
         $lon = $deg + ($lon - $deg * 100) / 60;
-        my @a = unpack('C*', substr($$dataPt,$pos+22, 6));  # unpack date/time
+        my @a = unpack('C*', substr($$dataPt, $pos+22, 6)); # unpack date/time
         $a[0] += 2000;
         $et->HandleTag($tagTbl, GPSDateTime  => sprintf("%.4d:%.2d:%.2d %.2d:%.2d:%.2dZ", @a));
         $et->HandleTag($tagTbl, GPSLatitude  => $lat);
@@ -1491,6 +1478,129 @@ sub Process_gps0($$$)
     return 1;
 }
 
+#------------------------------------------------------------------------------
+# Process DuDuBell M1 dashcam / VSYS M6L 'gsen' atom (ref PH)
+# Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success
+sub Process_gsen($$$)
+{
+    my ($et, $dirInfo, $tagTbl) = @_;
+    my $dataPt = $$dirInfo{DataPt};
+    my $dirLen = $$dirInfo{DirLen};
+    my $recLen = 3;     # 3-byte record length
+    $et->VerboseDir('gsen', undef, $dirLen);
+    if ($dirLen > $recLen and not $et->Options('ExtractEmbedded')) {
+        $dirLen = $recLen;
+        EEWarn($et);
+    }
+    my $pos;
+    for ($pos=0; $pos+$recLen<=$dirLen; $pos+=$recLen) {
+        $$et{DOC_NUM} = ++$$et{DOC_COUNT};
+        my @acc = map { $_ /= 16 } unpack "x${pos}c3", $$dataPt;
+        $et->HandleTag($tagTbl, Accelerometer => "@acc");
+        # (there are no associated timestamps, but these are sampled at 5 Hz in my test video)
+    }
+    delete $$et{DOC_NUM};
+    return 1;
+}
+
+#------------------------------------------------------------------------------
+# Process RIFF-format trailer written by Auto-Vox dashcam (ref PH)
+# Inputs: 0) ExifTool ref, 1) dirInfo ref, 2) tag table ref
+# Returns: 1 on success
+# Note: This trailer is basically RIFF chunks added to a QuickTime-format file (augh!),
+#       but there are differences in the record formats so we can't just call
+#       ProcessRIFF to process the gps0 and gsen atoms using the routines above
+sub ProcessRIFFTrailer($$$)
+{
+    my ($et, $dirInfo, $tagTbl) = @_;
+    my $raf = $$dirInfo{RAF};
+    my $verbose = $et->Options('Verbose');
+    my ($buff, $pos);
+    SetByteOrder('II');
+    for (;;) {
+        last unless $raf->Read($buff, 8) == 8;
+        my ($tag, $len) = unpack('a4V', $buff);
+        last if $tag eq "\0\0\0\0";
+        unless ($tag =~ /^[\w ]{4}/ and $len < 0x2000000) {
+            $et->Warn('Bad RIFF trailer');
+            last;
+        }
+        $raf->Read($buff, $len) == $len or $et->Warn("Truncated $tag record in RIFF trailer"), last;
+        if ($verbose) {
+            $et->VPrint(0, "  - RIFF trailer '${tag}' ($len bytes)\n");
+            $et->VerboseDump(\$buff, Addr => $raf->Tell() - $len) if $verbose > 2;
+            $$et{INDENT} .= '| ';
+            $et->VerboseDir($tag, undef, $len) if $tag =~ /^(gps0|gsen)$/;
+        }
+        if ($tag eq 'gps0') {
+            # (similar to record decoded in Process_gps0, but with some differences)
+            # 0000: 41 49 54 47 74 46 94 f6 c6 c5 b4 40 34 a2 b4 37 [AITGtF.....@4..7]
+            # 0010: f8 7b 8a 40 ff ff 00 00 38 00 77 0a 1a 0c 12 28 [.{.@....8.w....(]
+            # 0020: 8d 01 02 40 29 07 00 00                         [...@)...]
+            # 0x00 - undef[4] 'AITG'
+            # 0x04 - double   latitude  (always positive)
+            # 0x0c - double   longitude (always positive)
+            # 0x14 - ?        seen hex "ff ff 00 00" (altitude in Process_gps0 record below)
+            # 0x18 - int16u   speed in knots (different than km/hr in Process_gps0)
+            # 0x1a - int8u[6] yr-1900,mon,day,hr,min,sec (different than -2000 in Process_gps0)
+            # 0x20 - int8u    direction in degrees / 2
+            # 0x21 - int8u    guessing that this is 1=N, 2=S - PH
+            # 0x22 - int8u    guessing that this is 1=E, 2=W - PH
+            # 0x23 - ?        seen hex "40"
+            # 0x24 - in32u    time since start of video (ms)
+            my $recLen = 0x28;
+            for ($pos=0; $pos+$recLen<$len; $pos+=$recLen) {
+                substr($buff, $pos, 4) eq 'AITG' or $et->Warn('Unrecognized gps0 record'), last;
+                $$et{DOC_NUM} = ++$$et{DOC_COUNT};
+                # lat/long are in DDDMM.MMMM format
+                my $lat = GetDouble(\$buff, $pos+4);
+                my $lon = GetDouble(\$buff, $pos+12);
+                $et->Warn('Bad gps0 record') and last if abs($lat) > 9000 or abs($lon) > 18000;
+                my $deg = int($lat / 100);
+                $lat = $deg + ($lat - $deg * 100) / 60;
+                $deg = int($lon / 100);
+                $lon = $deg + ($lon - $deg * 100) / 60;
+                $lat = -$lat if Get8u(\$buff, $pos+0x21) == 2;   # wild guess
+                $lon = -$lon if Get8u(\$buff, $pos+0x22) == 2;   # wild guess
+                my @a = unpack('C*', substr($buff, $pos+26, 6)); # unpack date/time
+                $a[0] += 1900; # (different than Proces_gps0)
+                $et->HandleTag($tagTbl, SampleTime => Get32u(\$buff, $pos + 0x24) / 1000);
+                $et->HandleTag($tagTbl, GPSDateTime  => sprintf("%.4d:%.2d:%.2d %.2d:%.2d:%.2dZ", @a));
+                $et->HandleTag($tagTbl, GPSLatitude  => $lat);
+                $et->HandleTag($tagTbl, GPSLongitude => $lon);
+                $et->HandleTag($tagTbl, GPSSpeed     => Get16u(\$buff, $pos+0x18) * $knotsToKph);
+                $et->HandleTag($tagTbl, GPSSpeedRef  => 'K');
+                $et->HandleTag($tagTbl, GPSTrack     => Get8u(\$buff, $pos+0x20) * 2);
+                $et->HandleTag($tagTbl, GPSTrackRef  => 'T');
+            }
+        } elsif ($tag eq 'gsen') {
+            # (similar to record decoded in Process_gsen)
+            # 0000: 41 49 54 53 1a 0d 05 ff c8 00 00 00 [AITS........]
+            # 0x00 - undef[4] 'AITS'
+            # 0x04 - int8s[3] accelerometer readings
+            # 0x07 - ?        seen hex "ff"
+            # 0x08 - in32u    time since start of video (ms)
+            my $recLen = 0x0c;
+            for ($pos=0; $pos+$recLen<$len; $pos+=$recLen) {
+                substr($buff, $pos, 4) eq 'AITS' or $et->Warn('Unrecognized gsen record'), last;
+                $$et{DOC_NUM} = ++$$et{DOC_COUNT};
+                my @acc = map { $_ /= 24 } unpack('x'.($pos+4).'c3', $buff);
+                $et->HandleTag($tagTbl, SampleTime => Get32u(\$buff, $pos + 8) / 1000);
+                # 0=+Up, 1=+Right, 3=+Forward (calibration of 24 counts/g is a wild guess - PH)
+                $et->HandleTag($tagTbl, Accelerometer => "@acc");
+            }
+        }
+        # also seen, but not decoded:
+        # gpsa (8 bytes): hex "01 20 00 00 08 03 02 08 "  
+        # gsea (20 bytes): all zeros
+        $$et{INDENT} = substr($$et{INDENT}, 0, -2) if $verbose;
+    }
+    delete $$et{DOC_NUM};
+    SetByteOrder('MM');
+    return 1;
+}
+
 #------------------------------------------------------------------------------
 # Process 'gps ' atom containing NMEA from Pittasoft Blackvue dashcam (ref PH)
 # Inputs: 0) ExifTool object ref, 1) dirInfo ref, 2) tag table ref
@@ -1652,7 +1762,7 @@ sub ProcessTTAD($$$)
 }
 
 #------------------------------------------------------------------------------
-# Extract information from Insta360 trailer
+# Extract information from Insta360 trailer (ref PH)
 # Inputs: 0) ExifTool ref
 sub ProcessINSVTrailer($)
 {
@@ -1667,23 +1777,29 @@ sub ProcessINSVTrailer($)
     my $tagTbl = GetTagTable('Image::ExifTool::QuickTime::Stream');
     my $fileEnd = $raf->Tell();
     my $trailerLen = unpack('x38V', $buff);
-    my $trailerStart = $fileEnd - $trailerLen;
+    $trailerLen > $fileEnd and $et->Warn('Bad INSV trailer size'), return;
     my $unknown = $et->Options('Unknown');
     my $verbose = $et->Options('Verbose');
-    my $pos = $fileEnd - 78;
+    my $epos = -78;   # position relative to EOF (avoids using large offsets for files > 2 GB)
     my ($i, $p);
     SetByteOrder('II');
     # loop through all records in the trailer, from last to first
     for (;;) {
         my ($id, $len) = unpack('vV', $buff);
-        ($pos -= $len) < $trailerStart and last;
-        $raf->Seek($pos, 0) or last;
-        $raf->Read($buff, $len) == $len or last;
-        if ($verbose) {
-            $et->VPrint(0, sprintf("INSV Record 0x%x (offset 0x%x, %d bytes):\n", $id, $pos, $len));
-            $et->VerboseDump(\$buff);
-        }
+        ($epos -= $len) + $trailerLen < 0 and last;
+        $raf->Seek($epos, 2) or last;
         my $dlen = $insvDataLen{$id};
+        if ($verbose) {
+            $et->VPrint(0, sprintf("INSV Record 0x%x (offset 0x%x, %d bytes):\n", $id, $fileEnd + $epos, $len));
+        }
+        # limit the number of records we read if necessary
+        if ($insvLimit{$id} and $len > $insvLimit{$id}[1] * $dlen and
+            $et->Warn("INSV $insvLimit{$id}[0] data is huge. Processing only the first $insvLimit{$id}[1] records",2))
+        {
+            $len = $insvLimit{$id}[1] * $dlen;
+        }
+        $raf->Read($buff, $len) == $len or last;
+        $et->VerboseDump(\$buff) if $verbose > 2;
         if ($dlen) {
             $len % $dlen and $et->Warn(sprintf('Unexpected INSV record 0x%x length',$id)), last;
             if ($id == 0x300) {
@@ -1719,7 +1835,7 @@ sub ProcessINSVTrailer($)
                     $et->HandleTag($tagTbl, GPSTrack => $a[9]);
                     $et->HandleTag($tagTbl, GPSTrackRef => 'T');
                     $et->HandleTag($tagTbl, GPSAltitude => $a[10]);
-                    $et->HandleTag($tagTbl, Unknown02 => "@a[1,2]");
+                    $et->HandleTag($tagTbl, Unknown02 => "@a[1,2]") if $unknown;
                 }
             }
         } elsif ($id == 0x101) {
@@ -1733,8 +1849,8 @@ sub ProcessINSVTrailer($)
                 $p += 2 + $n;
             }
         }
-        ($pos -= 6) < $trailerStart and last;   # step back to previous record
-        $raf->Seek($pos, 0) or last;
+        ($epos -= 6) + $trailerLen < 0 and last;    # step back to previous record
+        $raf->Seek($epos, 2) or last;
         $raf->Read($buff, 6) == 6 or last;
     }
     SetByteOrder('MM');
